local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

local AceGUI = LibStub("AceGUI-3.0")

local function PopulateDropDown(dropdown, tableValues, indexAsValue, callbackSelected)
	UIDropDownMenu_ClearAll(dropdown)
	UIDropDownMenu_Initialize(dropdown, function(self, level)
											for k, v in pairs(tableValues) do
												local text = v
												local value = k
												if (not indexAsValue) then
													value = v
												end

												local info = UIDropDownMenu_CreateInfo()
												info.text = text
												info.value = value
												info.func = function(self)
																UIDropDownMenu_SetSelectedValue(dropdown, self.value)
																if (callbackSelected) then
																	callbackSelected(self)
																end
															end
												UIDropDownMenu_AddButton(info, level)
											end
										end)
end

local function CallbackSelectedStackSize(self)
	Capitalist.auctionPostingFrame.postingParams.stackSize = self.value
	Capitalist.auctionPostingFrame.postingParams.numStacks = 1
	Capitalist:UpdatePostingControls()
end

local function CallbackSelectedStackCount(self)
	Capitalist.auctionPostingFrame.postingParams.numStacks = self.value
	Capitalist:UpdatePostingControls()
end

local function CallbackSelectedDuration(self)
	Capitalist.auctionPostingFrame.postingParams.time = self.value
	Capitalist:UpdatePostingControls()
end

local function CallbackPost(self)
	local postingParams = Capitalist.auctionPostingFrame.postingParams
	Capitalist.PostingPostCountAfterCreated = postingParams.numAvailable - (postingParams.stackSize * postingParams.numStacks)
	if (postingParams.numStacks > 0) then
		StartAuction(
			postingParams.bid * postingParams.stackSize,
			postingParams.buyout * postingParams.stackSize,
			postingParams.time,
			postingParams.stackSize,
			postingParams.numStacks
			)
	else
		Capitalist:PostAborted(true)
		Capitalist:PostAttemptDone(true)
	end
end

local function CallbackSkip(self)
	Capitalist:PostAborted(true)
	Capitalist:PostAttemptDone(true)
end

local function CallbackStop(self)
	Capitalist:PostCallback(false)
end

-- TODO: Trim out the data population from this function, since Update overrides it anyway
function Capitalist:CreateAuctionPostingFrame()
	--local itemLink = select(2, GetItemInfo(GetAuctionSellItemInfo()))
	local itemLink = "|cffffffff|Hitem:43423:0:0:0:0:0:0:0:80:0|h[Glyph of Slam]|h|r"
	local deposit = 555555
	local bid = 777777
	local buyout = 999999
	-- TODO: 3 = 48 hour, hardcoded
	local time = 3
	local stackSize = 5
	local stackSizeMax = 20
	local numStacksRecommended = 2
	local numStacksAvailable = 3

	local parent = nil

	local container = CreateFrame("Frame", "CapitalistAuctionPostingFrame", parent)
	container:SetScale(0.5)
	container:SetPoint("TOP", parent, "TOP", 100, 0)
	container:SetWidth(400)
	container:SetHeight(150)
	container:SetFrameStrata("HIGH")

	container.background = container:CreateTexture(nil, "BACKGROUND")
	container.background:SetAllPoints()
	container.background:SetTexture(0,0,0)

	local backdrop = {
	  --bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	  edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
	  edgeSize = 26,
	  insets = {left = 9, right = 9, top = 9, bottom = 9},
	}
	container:SetBackdrop(backdrop)

	container.labelItem = container:CreateFontString(nil, nil, "GameFontNormalLarge")
	container.labelItem:SetPoint("TOP", container, "TOP", 0, -20)
	container.labelItem:SetText(itemLink)
	container.labelItem:SetWidth(350)

	container.labelDeposit = container:CreateFontString(nil, nil, "GameFontNormal")
	container.labelDeposit:SetPoint("BOTTOMLEFT", container, "BOTTOMLEFT", 20, 80)
	container.labelDeposit:SetTextColor(1, 1, 1)
	container.labelDeposit:SetText(string.format(L["|cffff8080Deposit|r: %s"], self:PrettyCurrency(deposit)))
	container.labelDeposit:SetJustifyH("LEFT")
	container.labelDeposit:SetWidth(200)

	container.labelBuyout = container:CreateFontString(nil, nil, "GameFontNormal")
	container.labelBuyout:SetPoint("BOTTOMRIGHT", container, "BOTTOMRIGHT", -20, 80)
	container.labelBuyout:SetTextColor(1, 1, 1)
	container.labelBuyout:SetText(string.format(L["|cff80ff80Buyout|r: %s"], self:PrettyCurrency(buyout)))
	container.labelBuyout:SetJustifyH("RIGHT")
	container.labelBuyout:SetWidth(200)

	container.labelBid = container:CreateFontString(nil, nil, "GameFontNormal")
	container.labelBid:SetPoint("BOTTOMRIGHT", container, "BOTTOMRIGHT", -20, 95)
	container.labelBid:SetTextColor(1, 1, 1)
	container.labelBid:SetText(string.format(L["|cff80ff80Bid|r: %s"], self:PrettyCurrency(bid)))
	container.labelBid:SetJustifyH("RIGHT")
	container.labelBid:SetWidth(200)

	if (bid == buyout) then
		container.labelBid:SetAlpha(0.5)
	else
		container.labelBid:SetAlpha(1)
	end

	container.dropdownStackSize = CreateFrame("Frame", "CapitalistDropDownStackSize", container, "UIDropDownMenuTemplate")
	local itemsStackSize = {}
	for size=1, stackSizeMax do
		table.insert(itemsStackSize, size)
	end
	PopulateDropDown(container.dropdownStackSize, itemsStackSize, false, CallbackSelectedStackSize)
	UIDropDownMenu_SetWidth(container.dropdownStackSize, 80)
	UIDropDownMenu_SetButtonWidth(container.dropdownStackSize, 100)
	--UIDropDownMenu_SetSelectedID(container.dropdownStackSize, stackSize)
	UIDropDownMenu_SetSelectedValue(container.dropdownStackSize, stackSize)
	UIDropDownMenu_JustifyText(container.dropdownStackSize, "RIGHT")
	container.dropdownStackSize:SetPoint("BOTTOMLEFT", container, "BOTTOMLEFT", 20, 30)


	container.dropdownStackCount = CreateFrame("Frame", "CapitalistDropDownStackCount", container, "UIDropDownMenuTemplate")
	local itemsStackCount = {}
	for size=1, numStacksAvailable do
		table.insert(itemsStackCount, size)
	end
	PopulateDropDown(container.dropdownStackCount, itemsStackCount, false, CallbackSelectedStackCount)
	UIDropDownMenu_SetWidth(container.dropdownStackCount, 80)
	UIDropDownMenu_SetButtonWidth(container.dropdownStackCount, 100)
--	UIDropDownMenu_SetSelectedValue(container.dropdownStackCount, numStacksRecommended)
	UIDropDownMenu_JustifyText(container.dropdownStackCount, "RIGHT")
	container.dropdownStackCount:SetPoint("BOTTOM", container, "BOTTOM", 0, 30)

	container.dropdownDuration = CreateFrame("Frame", "CapitalistDropDownDuration", container, "UIDropDownMenuTemplate")
	local itemsDuration = { AUCTION_DURATION_ONE, AUCTION_DURATION_TWO, AUCTION_DURATION_THREE, }

	PopulateDropDown(container.dropdownDuration, itemsDuration, true, CallbackSelectedDuration)
	UIDropDownMenu_SetWidth(container.dropdownDuration, 80)
	UIDropDownMenu_SetButtonWidth(container.dropdownDuration, 100)
--	UIDropDownMenu_SetSelectedID(container.dropdownDuration, 1)
	UIDropDownMenu_JustifyText(container.dropdownDuration, "RIGHT")
	container.dropdownDuration:SetPoint("BOTTOMRIGHT", container, "BOTTOMRIGHT", -20, 30)

	container.labelStackSize = container:CreateFontString(nil, nil, "GameFontNormal")
	container.labelStackSize:SetPoint("BOTTOM", container.dropdownStackSize, "TOP")
	container.labelStackSize:SetText(L["Stack size"])

	container.labelStackCount = container:CreateFontString(nil, nil, "GameFontNormal")
	container.labelStackCount:SetPoint("BOTTOM", container.dropdownStackCount, "TOP")
	container.labelStackCount:SetText(L["Stack count"])

	container.labelDuration = container:CreateFontString(nil, nil, "GameFontNormal")
	container.labelDuration:SetPoint("BOTTOM", container.dropdownDuration, "TOP")
	container.labelDuration:SetText(L["Duration"])

	container.buttonPost = CreateFrame("Button", nil, container, "UIPanelButtonTemplate")
	container.buttonPost:SetWidth(100)
	container.buttonPost:SetHeight(20)
	container.buttonPost:SetText(L["Post item(s)"])
	container.buttonPost:SetPoint("BOTTOMLEFT", container, "BOTTOMLEFT", 10, 10)
	container.buttonPost:SetScript("OnClick", CallbackPost)

	container.buttonSkip = CreateFrame("Button", nil, container, "UIPanelButtonTemplate")
	container.buttonSkip:SetWidth(100)
	container.buttonSkip:SetHeight(20)
	container.buttonSkip:SetText(L["Skip item(s)"])
	container.buttonSkip:SetPoint("BOTTOM", container, "BOTTOM", 0, 10)
	container.buttonSkip:SetScript("OnClick", CallbackSkip)

	container.buttonStop = CreateFrame("Button", nil, container, "UIPanelButtonTemplate")
	container.buttonStop:SetWidth(100)
	container.buttonStop:SetHeight(20)
	container.buttonStop:SetText(L["Stop posting"])
	container.buttonStop:SetPoint("BOTTOMRIGHT", container, "BOTTOMRIGHT", -10, 10)
	container.buttonStop:SetScript("OnClick", CallbackStop)

	return container
end

function Capitalist:HideAuctionPostingFrame()
	if (self.auctionPostingFrame) then
		self.auctionPostingFrame:Hide()
	end
end

local function BuildNumericChoiceTable(maxValue)
	local tableValues = {}
	for size=0, math.min(20, maxValue) do
		table.insert(tableValues, size)
	end
	for size=30, math.min(90, maxValue), 10 do
		table.insert(tableValues, size)
	end
	for size=100, math.min(500, maxValue), 50 do
		table.insert(tableValues, size)
	end
	for size=600, math.min(900, maxValue), 100 do
		table.insert(tableValues, size)
	end
	for size=1000, maxValue, 1000 do
		table.insert(tableValues, size)
	end
	return tableValues
end

function Capitalist:UpdatePostingControls()
	local itemName, _, _, _, _, _, _, itemStackCount, _, itemTexture, itemSellPrice  = GetItemInfo(self.auctionPostingFrame.postingParams.itemLink)

	local zeroProduct = ((self.auctionPostingFrame.postingParams.stackSize <= 0) or (self.auctionPostingFrame.postingParams.numStacks <= 0))

	local deposit = 0
	if (not zeroProduct) then
		deposit = CalculateAuctionDeposit(
						self.auctionPostingFrame.postingParams.time,
						self.auctionPostingFrame.postingParams.stackSize,
						1
					)
	end

	local insufficientFunds = (deposit * self.auctionPostingFrame.postingParams.numStacks > GetMoney())
	if (zeroProduct or insufficientFunds) then
		self.auctionPostingFrame.buttonPost:Disable()
	else
		self.auctionPostingFrame.buttonPost:Enable()
	end

	self.auctionPostingFrame.labelItem:SetText(self.auctionPostingFrame.postingParams.itemLink)

	local buyoutStack = self.auctionPostingFrame.postingParams.stackSize * self.auctionPostingFrame.postingParams.buyout
	local bidStack = self.auctionPostingFrame.postingParams.stackSize * self.auctionPostingFrame.postingParams.bid

	self.auctionPostingFrame.labelDeposit:SetText(string.format(L["|cffff8080Deposit|r: %s"], Capitalist:PrettyCurrency(deposit)))
	self.auctionPostingFrame.labelBuyout:SetText(string.format(L["|cff80ff80Buyout|r: %s"], Capitalist:PrettyCurrency(buyoutStack)))
	self.auctionPostingFrame.labelBid:SetText(string.format(L["|cff80ff80Bid|r: %s"], Capitalist:PrettyCurrency(bidStack)))

	if (self.auctionPostingFrame.postingParams.bid == self.auctionPostingFrame.postingParams.buyout) then
		self.auctionPostingFrame.labelBid:SetAlpha(0.5)
	else
		self.auctionPostingFrame.labelBid:SetAlpha(1)
	end

	local entriesStackSize = BuildNumericChoiceTable(math.min(itemStackCount, self.auctionPostingFrame.postingParams.numAvailable))
	PopulateDropDown(self.auctionPostingFrame.dropdownStackSize, entriesStackSize, false, CallbackSelectedStackSize)
	UIDropDownMenu_SetSelectedValue(self.auctionPostingFrame.dropdownStackSize, self.auctionPostingFrame.postingParams.stackSize)

	local entriesStackCount
	local maxStackCount = math.floor(self.auctionPostingFrame.postingParams.numAvailable / self.auctionPostingFrame.postingParams.stackSize)
	if (maxStackCount > 0) then
		entriesStackCount = BuildNumericChoiceTable(maxStackCount)
	else
		entriesStackCount = {}
	end
	if (entriesStackCount[#entriesStackCount] < maxStackCount) then
		table.insert(entriesStackCount, maxStackCount)
	end
--	REVIEW: To enable this, at least two changes:
--			1. handle string value selected
--			2. do a double-calculation + summation for deposit cost
--	local partialStackExtra = self.auctionPostingFrame.postingParams.numAvailable - (self.auctionPostingFrame.postingParams.stackSize * maxStackCount)
--	if (partialStackExtra > 0) then
--		table.insert(entriesStackCount, string.format("%d+ (All)", maxStackCount))
--	end

	PopulateDropDown(self.auctionPostingFrame.dropdownStackCount, entriesStackCount, false, CallbackSelectedStackCount)
	UIDropDownMenu_SetSelectedValue(self.auctionPostingFrame.dropdownStackCount, self.auctionPostingFrame.postingParams.numStacks)

	-- REVIEW: I tried bypassing recreating this Combo box and weird stuff happened like "Custom" as the setting or "2".
	-- TODO: I think I fixed that problem; verify in future build.
	local itemsDuration = { AUCTION_DURATION_ONE, AUCTION_DURATION_TWO, AUCTION_DURATION_THREE, }
	PopulateDropDown(self.auctionPostingFrame.dropdownDuration, itemsDuration, true, CallbackSelectedDuration)
	UIDropDownMenu_SetSelectedID(self.auctionPostingFrame.dropdownDuration, self.auctionPostingFrame.postingParams.time)
end

function Capitalist:UpdateAndShowAuctionPostingFrame(...)
	local itemLink, bid, buyout, time, stackSize, numStacks, numAvailable = ...

	-- REVIEW: Consider adding the itemTexture to brighten up the place [itemTexture]
	-- REVIEW: Consider tinting the background reddish if the itemSellPrice >= (bid - deposit) [itemSellPrice]
	local itemName, _, _, _, _, _, _, itemStackCount, _, itemTexture, itemSellPrice  = GetItemInfo(itemLink)

	local postingParams = {
		itemLink = itemLink,
		bid = bid,
		buyout = buyout,
		time = time,
		stackSize = stackSize,
		numStacks = numStacks,
		numAvailable = numAvailable,
	}

	if (not self.auctionPostingFrame) then
		self.auctionPostingFrame = Capitalist:CreateAuctionPostingFrame()
	end
	
	self.auctionPostingFrame.postingParams = postingParams;	
	self:UpdatePostingControls()

	self.auctionPostingFrame:Show()
end
